package multimedia.generation;

/**
 * MGInfAggregatedFlowGeneration  Generation de trafic video pour flowsNumber sources MGInf => flux aggreges
 *                                				 Ces flux peuvent etre differents ou non: par leur duree, leurs caracteristiques => sources heterogenes/homogenes.
 * 
 * 	For each flows: we define a:<br>
 * 		<ul><li> M/G/Infinity model
 * 		<li> Video codec
 * 		<li> a number of source (called here weight)
 * 		<li> a "life" duration
 * 		<li> for each source of this flow a starting time (it could be randomly choosen)
 * 		<li> a unit
 * 	</ul>
 * 
 * <p>
 * In theory, we could choose whatever distribution of data in slot (called distributionSlot) you want (ex: constant, poisson, pareto...) : here we assume that the distibution will be a constant of parameter 1 distribution ( because we couldn't manage different kind of distribution).<br>
 * Moreover, we could choose a whatever distribution of data in slot (called distributionSlot) you want (ex: constant, poisson, pareto...) : here we assume  that the distibution will be a constant distribution (because we couldn't manage different kind of distribution) but we could change packetSize
 * </p>
 *   
 * @author mladoue
 * @version 0.1
 */

import java.io.*;
import java.util.*;

import statistic.*;
import statistic.distribution.*;
import utils.*;
import utils.definitions.*;
import multimedia.codec.*;
import multimedia.model.*;
import multimedia.*;


public class MGInfAggregatedFlowGeneration extends GenericGeneration implements RandomGenerator, AudioCodecExample, VideoExample, VideoCodecExample {

	protected int[] flowsDuration;	// it corresponds to the tpsSimul of each flow => in mn
	protected VideoCodec[] codecs;
	protected MGInfAggregatedFlowModel model;
	protected short[] units;  // it correspond sto the unit (bytes, kiloBytes... of each flow type) : default is BYTES
	//	we give in ms(it's a choice: we could not have less than 1 ms between 2 flows) the time since To : it corresponds to the moment when a flow starts
	// We consider that first defined flow start the first.... =>  startingTime[0] = 0 (i.e To)
	protected int[] startingTime;	// in ms : you specify the moment when the flow will start (after To)
	protected int[] slotsNumber;	// slots number of each model
	protected double[] slotsDuration;  // duration of a slot for each model (in seconds)
	protected int nbDifferentFlow = 0;  // it's M
	protected int flowsNumber =0;  // it's L => L= W1 + W2 + ... WM
	protected int[] packetSize;  // packet size for each flow type
	protected MGInfGeneration[] mginfGeneration;
	
	// Characteristics of the new flow
	protected int simulationDuration = 0; // simulation duration: "in ms" because of starting time is defined in ms (equal to tpsSimul)
	protected  int N = 0; // slots number for the new aggregated flow!!!!
	protected  double slotDuration = 0d;  // theoritical duration of a slot for aggregated flow in seconds!
	protected double realRate = 0d;  
	
	protected File[] flowsFile;
	protected TreeMap[] tM;



	public MGInfAggregatedFlowGeneration ( ) {
		this.graphic = false;
		this.unit = BYTES;
	}
	
	/**
	 * @param model
	 * @param codecs	a table of VideoCodec => length :  nbDifferentFlow
	 * @param flowsDuration	a table of int => length :  nbDifferentFlow
	 * @param startingTime	a table of int => length :  flowsNumber
	 * @param units	a table of short => length :  nbDifferentFlow
	 * @param fileName	Scenario file name
	 * @param graphic
	 * @throws IOException
	 */	
	public MGInfAggregatedFlowGeneration ( MGInfAggregatedFlowModel model, VideoCodec[] codecs, int[] flowsDuration, int[] startingTime, short[] units,  String fileName, boolean graphic)  throws IOException {
		
		this.nbDifferentFlow = model.getFlowTypeNumber();//models.length;
		this.graphic = graphic;
		this.fileName = fileName;
		
		if ( nbDifferentFlow == flowsDuration.length ) {
			this.model = model;
			this.codecs = codecs;
			this.flowsDuration = flowsDuration;
			this.flowsNumber = model.getSourceNumber();
	
			if ( startingTime.length == flowsNumber) {
				this.startingTime = startingTime;
			}
			else {
				throw new IllegalArgumentException("You have to specify a starting time for each flow: " + startingTime.length +  " instead of " + flowsNumber );
			}
			
			parametersInitialization();
		}
		else {
			throw new IllegalArgumentException("You have to specify for each flow: the number of source, the model, the codec and there duration!, there are: "  + codecs.length  + " codecs , " + flowsDuration.length + " specified durations.");
		}		
	}
	
	/**
	 * Defines to be used by MGInfComplexAggregatedFlowGeneration as super constructor
	 * @param vds
	 * @param weights
	 * @param unit
	 * @param fileName	Scenario file name
	 * @param graphic
	 * @throws IOException
	 */
	public MGInfAggregatedFlowGeneration ( VideoDefinition[] vds, int[] weights, short unit, String fileName, boolean graphic)  throws IOException {
		
			this.nbDifferentFlow = vds.length;
			this.graphic = graphic;
			this.fileName = fileName;
		
			if ( nbDifferentFlow == weights.length  ) {
				this.model = new MGInfAggregatedFlowModel( new MGInfModel[vds.length], weights);
				this.codecs = new VideoCodec[vds.length];
				this.flowsDuration = new int[vds.length];
				this.units = new short[vds.length];
				
				this.flowsNumber = model.getSourceNumber();
				this.startingTime = new int[flowsNumber];
		
				for (int i=0;i<vds.length;i++) {	
					model.setSpecificModel(i, vds[i].getModel());
					codecs[i] = vds[i].getCodec();
					flowsDuration[i] = vds[i].getTpsSimul();
					units[i] = unit;
				}
	
				setSourceStartingTime( this.createArbitraryStartingTimeTable() );
		
				parametersInitialization();
			}
			else {
				throw new IllegalArgumentException("You have to specify for each flow: the number of source, the model, the codec and its duration! : " + weights.length +  " , " + vds.length + " , " + units.length  + " , " + startingTime.length);
			}	
	}
	
	/**
	 * Defined to used VideoDefinition like JurassicParkLow,...
	 * @param vds	a table of VideoDefinition => length :  nbDifferentFlow
	 * @param weights	a table of int => length :  nbDifferentFlow
	 * @param startingTime	a table of int => length :  flowsNumber
	 * @param units	a table of short => length :  nbDifferentFlow
	 * @param fileName	Scenario file name
	 * @param graphic
	 * @throws IOException
	 */
	public MGInfAggregatedFlowGeneration ( VideoDefinition[] vds, int[] weights, int[] startingTime, short[] units, String fileName, boolean graphic)  throws IOException {
		
		this.nbDifferentFlow = vds.length;
		this.graphic = graphic;
		this.fileName = fileName;
		
		if ( nbDifferentFlow == weights.length  && nbDifferentFlow == units.length ) {

			this.model = new MGInfAggregatedFlowModel( new MGInfModel[vds.length], weights);
			this.codecs = new VideoCodec[vds.length];
			this.flowsDuration = new int[vds.length];
		
			this.flowsNumber = model.getSourceNumber();
			this.units = units;
		
			for (int i=0;i<vds.length;i++) {	
				model.setSpecificModel(i, vds[i].getModel());
				codecs[i] = vds[i].getCodec();
				flowsDuration[i] = vds[i].getTpsSimul();
			}
	
			if ( startingTime.length == flowsNumber) { this.startingTime = startingTime; }
			else { throw new IllegalArgumentException("You have to specify a starting time for each flow: " + startingTime.length +  " instead of " + flowsNumber );	}
		
			parametersInitialization();
		}
		else {
			throw new IllegalArgumentException("You have to specify for each flow: the number of source, the model, the codec and its duration! : " + weights.length +  " , " + vds.length + " , " + units.length  + " , " + startingTime.length);
		}		
	}
	

	/**
	 * This permit to initialized some parameters!!! 
	 * @throws IOException
	 */
	public void parametersInitialization() throws IOException {
		
		unit = BYTES;  // on doit tout ramener a des BYTES => a la fin tout sera en Bytes
		mginfGeneration = new MGInfGeneration[nbDifferentFlow];
		slotsNumber = new int[nbDifferentFlow];
		slotsDuration = new double[nbDifferentFlow];
		
		createSingleFlowGenerator();	 // permit to create one MGInfGeneration handler per flow type 
		setSlotsNumber();  // to initialized slotsNumber	
		setSlotsDuration();	// to initialized slotsDuration	
		slotDuration = TestParameters.getMinimumDoubleValue( slotsDuration );  // in seconds;
		simulationDuration = simulationDurationComputation();  // in ms!!!!
		tpsSimul = Math.round(simulationDuration/1000/60);  // transform ms in mn
		setSlotNumber();  // you need to have initialized: simulationDuration and slotDuration		
		setPacketSize();

		//System.out.println("Duration: " + simulationDuration);
	}
	
	

	/**
	 * Arbitrary method: we choose that flows will start with 2 seconds of distance
	 * 	To<i>i<\i> = To + i*2s 
	 * @return
	 */
	protected int[] createArbitraryStartingTimeTable()  {
		int[] ans = new int[flowsNumber];
		for (int i=0;i<flowsNumber;i++) {
			ans[i] = i*2*1000;	
		}
		return ans;
	}
	
	/**
	 * To set the number of slots of each source type 
	 */
	public void setSlotsNumber() {
		for (int i=0;i<nbDifferentFlow;i++) {
			slotsNumber[i] = mginfGeneration[i].slotsNumberComputation();
		}
	}
	
	/**
	 * To set the duration of a slot for each source type
	 */
	public void setSlotsDuration() {
		for ( int i=0;i<nbDifferentFlow;i++ ) {
			slotsDuration[i] = mginfGeneration[i].slotDurationComputation();
		}
	}
	
	/**
	 *  To set the number of slots for the aggregated flow => N
	 */
	protected void setSlotNumber() {
		this.N = (int) Math.round( simulationDuration /(slotDuration*1000)); 
	}
	
	
	/**
	 * Method createSingleFlowGenerator : To create MGInfGeneration object from model,flowsDuration,codecs...
	 * 													We could use MGInfGeneration.dataPerSlotDistributionTransformation,  MGInfGeneration.dataPerSlotDistributionComputation methods
	 * @return	MGInfGeneration[] :	as many MGInfGeneration object as nbDifferentFlow 
	 * @throws IOException
	 */
	public MGInfGeneration[] createSingleFlowGenerator() throws IOException {
			for (int i=0;i<nbDifferentFlow;i++) {
				mginfGeneration[i] = new MGInfGeneration( model.getSpecificModel(i), flowsDuration[i], codecs[i],1);
				mginfGeneration[i].setUnit(units[i]);
				mginfGeneration[i].setGraphicMode(graphic);
			}
			return mginfGeneration;
	}
	
	/**
	 * Method simulationDurationComputation : To compute the duration of the aggregated flow according to "starting time" scenario  
	 * @return the simultation duration in ms
	 */
	public int simulationDurationComputation() {
		int simulationDuration = 0;  // le resultat doit etre donne en ms (starting time est donne en ms)
		
		for (int i=0; i<nbDifferentFlow; i++) {
			for (int j=0;j<flowsNumber;j++) {
				if( simulationDuration < ( flowsDuration[i]*60*1000 + startingTime[j]) ) { // transforme en ms
					simulationDuration =  Math.max( simulationDuration, flowsDuration[i]*60000 + startingTime[j]);
				}
			}
		}
		return (int)Math.floor(simulationDuration); // on retourne un resultat en milli-secondes
	}	
	

	/**
	 * Method dataPerSlotPerFlowType : This method permit to compute data per slot distribution for one type of source
	 * 												After we transform the result according to unit and number of source of this type
	 * 												=> ex: if unit: kiloBytes -> Bytes and if there is 5 sources of this type data[i] = data[i]*5  
	 * @return a new time slots vector which represents the amount of data which has to be generated for this type of souce (i.e this flow)<br>
	 * 			size : [ nbDifferentFlow ] x [ slotsNumberI ]
	 * @throws IOException
	 */
	public int[][] dataPerSlotPerFlowType() throws IOException {
		int[][] data = new int [nbDifferentFlow][];  // size [ nbDifferentFlow ] x [ slotsNumberI ] <=> [ nbDifferentFlow ] x [ NI +1 ] 
		
		for ( int i=0; i<nbDifferentFlow; i++) {
			// we get the dataSlotDistribution for each type flow => but it represents only one source in the flow so we have to take weight of each flows into account
			data[i] = MGInfGeneration.dataPerSlotDistributionTransformation( model.getSpecificModel(i), slotsNumber[i] );

			for(int j=0;j<data[i].length;j++) {
				// Transform data amount for each flow according to its unit(bytes,...)
				// if unit is BYTES do nothing else tranform data amount unit in BYTES (kiloBytes -> Bytes or megaBytes -> Bytes)	
				if( units[i] != BYTES ) {
					data[i][j] *= Math.pow(10, 3*units[i]);
				}
			}
		}
		return data;
	}

	
	/**
	 * Transforms a flow<sub>i</sub> type definition([slot duration<sub>i</sub>, slot number<sub>i</sub>]) in order to obtain a flow representation as follow:
	 * 				([ reference slot duration, new slot number<sub>i</sub>])
	 * @return an int[][]  with size : [ nbDifferentFlow ] x [ newSlotsNumberI ] 
	 * @throws IOException
	 */
	protected int[][] slotDurationStandardization() throws IOException {
		
		int[][] newData = new int[nbDifferentFlow][];  // size : [ nbDifferentFlow ] x [ newSlotsNumberI ] 
		int[][] data = dataPerSlotPerFlowType();  // size : [ nbDifferentFlow ] x [ slotsNumberI ]
		//System.out.println(data[0].length +  " , "+ data[1].length  +  " , "+  data[2].length);	// DEBUG
		
		int slotDurationReference = getNewSlotDurationReference();  // value in ms
		int newSlotNumberI, slotEquivalentNumberI, cptOldSlot=0;
		
		for ( int i=0; i<nbDifferentFlow; i++) {
			newSlotNumberI = (int)Math.round( flowsDuration[i]*60*1000 / (double)(slotDurationReference) );  // transform in ms
			slotEquivalentNumberI = (int)Math.round( slotDurationReference/(double)(slotsDuration[i]*1000));
			newData[i] = new int[newSlotNumberI];	// IMPORTANT initialisation du ieme tableau
			
			/* Test new solution...
			for( int j=0;j<newSlotNumberI;j++ ) {
				while( j+slotEquivalentNumberI > cptOldSlot ) {
					newData[i][j] += data[i][cptOldSlot];
					cptOldSlot++;	
				}
			}
			cptOldSlot = 0;*/
			for( int j=0;j<newSlotNumberI;j++ ) {
				while( slotEquivalentNumberI > cptOldSlot ) {
					newData[i][j] += data[i][j*slotEquivalentNumberI + cptOldSlot];
					cptOldSlot++;	
				}
				cptOldSlot=0;
			}
		}
		return newData;
	}
	
	
	/**
	 * Transforms a source <sub>i</sub> definition([ reference slot duration, new slot number<sub>i</sub>]) in order to obtain a flow representation as follow:<br>
	 * 					([ reference slot duration, reference slot number])
	 * @return	an int[][]  with size : size : [  flowsNumber ] x [ newSlotNumber ]
	 * @throws IOException
	 */
	protected int[][] slotNumberStandardization() throws IOException {
		
			int slotDurationReference = getNewSlotDurationReference();	// value in ms
			int newSlotNumber = (int)Math.round(simulationDuration / (double)slotDurationReference);
			
			int[][] newData = new int[ flowsNumber ][newSlotNumber+1];  // size : [  flowsNumber ] x [ newSlotNumber ]
			int[][] data = slotDurationStandardization();	 // size : [ nbDifferentFlow ] x [ newSlotsNumberI ]		
			//System.out.println(data[0].length +  " , "+ data[1].length  +  " , "+  data[2].length);	// DEBUG
		
			int cptFlows = 0, cptOldSlots =0;
			int[] newStartingTime = transformStartingTimeInSlotsNumber( slotDurationReference );  // parameter must be in ms
			
			for ( int i=0; i<nbDifferentFlow; i++) {
				for(int i1 =0;i1< model.getSpecificWeight(i);i1++) {
					for( int j=0;j<newSlotNumber;j++ ) {
						if( newStartingTime[cptFlows] > j  || cptOldSlots >= data[i].length ) {
							newData[cptFlows][j] = 0;
						}
						else {
							newData[cptFlows][j] = data[i][cptOldSlots];
							cptOldSlots++;
						}			
					}
					cptFlows++;
					cptOldSlots =0;
				}
			}
			return newData;
		}
		
	/**
	* Sums all data amount of sources slot per slot
	* @return an array of int : size N
	*/
	 public int[] dataPerSlotDistribution() throws IOException {
	  	
		int[] dataAmount = new int[N+1]; 	// size : N
		int[][] data = slotNumberStandardization();	// size : [  flowsNumber ] x [ newSlotNumber ]
		int[] dataAmountTmp = new int[data[0].length];	// size :  [ newSlotNumber ]
		
		int cptSlot = 0, cpt=0;

			//	We create a temporary table to stock  sum of data of each flow with a slot number of newSlotNumber
			// 	slots vector characteristics: [ newSlotNumber, slotDurationReference] 
			for( int i=0;i<data[0].length;i++) {
				for (int j=0;j < flowsNumber;j++) {
					dataAmountTmp[i] +=  data[j][i];	
				}
			}

			// Transform the aggregated slot ("old" characteristics: [newSlotNumber, slotDurationReference] ) to obtain a slot vector with N slots of slotDuration duration ("new" characteristics: [ N, slotDuration] )
			double factor = N /(data[0].length-1);
			int newData =0;	
			for(int i=0; i< dataAmountTmp.length; i++) {
				newData = (int)Math.round( dataAmountTmp[i] / factor );
				while( cpt < factor ) {
					dataAmount[cptSlot] = newData;
					cptSlot++;
					cpt++;
				}
				cpt=0;
			}
			  return dataAmount;
		  }
		  
	
	
	/**
	 * Works a slot duration of "reference" out : this value corresponds to mcm of slotsDuration[i] 
	 * @return a slot "practical" slot duration!!!! in ms!!!
	 */
	protected  int getNewSlotDurationReference() {
		
		int slotDurationReference = 0;
		int[] tmp = new int[nbDifferentFlow];
		
		for(int i=0;i<nbDifferentFlow;i++) {
			tmp[i] = (int)Math.round(slotsDuration[i]*1000);
		}
		
		slotDurationReference = Mathematic.ppcmComplex(tmp);
		return slotDurationReference;
	}
	
	/**
	 * This method convert a table of starting time in ms in a table of value in number of slots
	 *	If slot duration = 10ms, 20 ms corresponds to 2 slots for example
	 * 
	 *	Here: startingTime it's a table which contains starting time of each source in ms
	 * @return	a table of int which corresponds to startingTime in slots
	 */
	protected int[] transformStartingTimeInSlotsNumber() {
		return transformStartingTimeInSlotsNumber(slotDuration*1000); // parameter is in ms
	}
	
	/**
	 * This method convert a table of starting time in: ms in a table of value in: number of slots
	 *	If slot duration = 10ms, 20 ms corresponds to 2 slots for example
	 *
	 * @param slotDurationReference it's given in seconds
	 * @return a table of int which corresponds to startingTime in slots
	 */
	protected int[] transformStartingTimeInSlotsNumber( double slotDurationReference ) {
		int[] newStartingTime = new int[startingTime.length];  // value in ms
		
		for (int i=0;i<flowsNumber;i++) {
			newStartingTime[i] = (int)Math.round(startingTime[i]/slotDurationReference);  // arrondit au-dessus
		}
		return newStartingTime;
	}	
	
	public int[] getFlowsPacketSize() { return packetSize; }
	
	/**
	 * To generate an aggregated scenario from an MGInfAggregatedFlowGeneration object
	 * And a scenario file per flow type	=> + nbDifferentFlow scenarii
	 * @throws IOException
	 */
	public void aggregatedScenarioGeneration() throws IOException {
			flowScenariiGeneration();
			scenarioGeneration();
	}
	
	/**
	 *  To generate an aggregated scenario with statistics informations from an MGInfAggregatedFlowGeneration object
	 *	And an informative scenario file per flow type => + nbDifferentFlow scenarii
	 * @param serverPort to specify on which port it will be played
	 * @throws IOException
	 */
	public void aggregatedStatisticsGeneration()  throws IOException {
		flowScenariiGeneration( );
		statisticsGeneration();
	}
	
	public double flowScenariiGeneration() throws IOException {

		tM = new TreeMap[nbDifferentFlow];
		flowsFile = new File[nbDifferentFlow];
	
		for (int i=0;i<nbDifferentFlow;i++) {
			tM[i] =  new TreeMap();
			flowsFile[i] = new File("./aggMGInf_"+ i);
			mginfGeneration[i].setResultFileName("./aggMGInf_"+ i);
			
			realRate += model.getSpecificWeight(i) * mginfGeneration[i].scenarioGenericGeneration(true);
			tM[i] = null;
		}
		return realRate;
	}
 
	/**
	 * Returns theoritical throughput of the aggregated flow => =sum( w1*<i>throughput<b>i</b></i>) where i =1...ndDifferentFlow <BR>
	 * Create <i>nbDifferentFlow</i> scenario files
	 * @param serverName
	 * @param serverPort
	 * @return throughput
	 * @throws IOException
	 */
	public double flowStatisticsiGeneration() throws IOException {

		tM = new TreeMap[nbDifferentFlow];
		flowsFile = new File[nbDifferentFlow];
	
		for (int i=0;i<nbDifferentFlow;i++) {
			tM[i] =  new TreeMap();
			flowsFile[i] = new File("./aggMGInf_"+ i);
			mginfGeneration[i].setResultFileName("./aggMGInf_"+ i);
			
			realRate += model.getSpecificWeight(i) * mginfGeneration[i].scenarioGenericGeneration( false);
			tM[i] = null;
		}
	    
		return realRate;
	}
 	
	/**
	 * @param playable	if it has to be a playable scenario or not
	 * @param serverName	server name
	 * @param serverPort	server port number
	 * @return average throughput in Bytes/s
	 * @throws IOException
	 */
	protected double scenarioGenericGeneration( boolean playable) throws IOException {
     	return scenarioGenericGeneration( dataPerSlotDistribution(), playable);
	}
	
	/**
	 * This method could be used by MGInfComplexAggregatedFlowGeneration
	 * 
	 * @param data	data distribution per slot ( in the form of a table)
	 * @param playable	if it has to be a playable scenario or not
	 * @return	throughput
	 * @throws IOException
	 */
	protected double scenarioGenericGeneration( int[] data, boolean playable) throws IOException {           
            
             if(!playable) {                	
				// Premiere ligne du scenario: date + nom_du_serveur
				fichierReq.write(  new Date().toString() + ".\n");
				fichierReq.write( "Requested documents : " + getDocumentNames() + ".\n");
				fichierReq.write( "Video aggregation \t Test Duration : " + (simulationDuration/1000/60)  +  "mn \n");
				fichierReq.write("Scenario format: date (in seconds) \t Inter-Packet Time(in milliseconds) \t packet size(in Bytes) \t  rate(in kb/s) \n\n" );// Format du fichier scenario
             }
              
		 //int[] slotNew = dataPerSlotDistribution();       
			int[] slotNew = data;
		
		  double time = 0; //en secondes
		  // Definit le nb de bytes deja envoye
		  int nbBytesTotal = 0; // en bytes
       
		  // For each slots we compute the number of packets, their size and their temporal distribution
		  for (int l = 0; l <= N; l++) {
            
           /**
            *  This permit to avoid capacity overflow => if it exceeds Integer.MAX_VALUE: 2147483647 around 2 gigas, we change to another unit
            */ 
		   if ( nbBytesTotal > (Integer.MAX_VALUE-slotNew[l]) && unit == BYTES) {
			unit = KILOBYTES;
			nbBytesTotal = (int)nbBytesTotal/1000;  // on passe de B->kB
			nbBytesTotal += (int)slotNew[l]/1000;	// on passe de B->kB
		   }
		   else if ( nbBytesTotal >(Integer.MAX_VALUE-slotNew[l]) && unit ==KILOBYTES) {
			unit = MEGABYTES;
			nbBytesTotal = (int)nbBytesTotal/1000;  // on passe de kB->MB
			nbBytesTotal += (int)slotNew[l]/1000000;	// on passe de B->MB
		   }
		   else if ( nbBytesTotal > (Integer.MAX_VALUE-slotNew[l]) && unit==MEGABYTES ) {
			unit = GIGABYTES;
			nbBytesTotal = (int)nbBytesTotal/1000;  // on passe de MB->GB
			nbBytesTotal += (int)slotNew[l]/1000000000;	// on passe de B->GB
		  }
		  else if ( unit==KILOBYTES || unit==MEGABYTES || unit==GIGABYTES) {
				nbBytesTotal += (int)slotNew[l]/1000;
		   }
		   else {
				nbBytesTotal += slotNew[l];
			}
   	
			  // Definit le nb de bytes deja envoye sur le slot courant
			  int nbBytes = 0; // en bytes
			  // Compte le nombre de paquets deja creer pour le slot courant
			  int counterPacket = 0;
			  // Time inter-packet, correspond to the result of dureeSlot/counterPacket or distributionsSlot.rand() (if the distribution is defined)
			  double IPTime = 0;
			  double startTimeSlot = time;
			  // Packet size in Bytes: it could be a constant or a distributionsPacketSize.rand()
			  int packet = 0;
			  // slot last packet size in Bytes => uniquement dans le cas d'une taille de paquet fixe
			  int lastPacket = 0;
			  // definit la taille de chacun des paquets qui sera genere a l'interieur d'un slot.
			  //Vector packetLength = new Vector(25);

			  if (slotNew[l] != 0) {

		/**
		 * WARNING:
		 * In this particular case:  packet distribution in a slot is constant and packet size is constant for a flow but not (necessarily) for the aggregation
		 * So we take flows weigth into account => we would select a flow and  
		 */

		  // Determine la taille des paquets a l'interieur d'un slot.
		 int transmitterSource = sourceTypeChoice();
		 packet = packetSize[transmitterSource-1];
		 
		 while (true) {
				 if ((packet + nbBytes) <= slotNew[l]) {
				 	counterPacket++;
				 	nbBytes += packet;
				} 
				 else {
				 	lastPacket = slotNew[l] - nbBytes;
				 	counterPacket++;
				 	nbBytes += lastPacket;
				 break;
				 }
		 }
 
	//	Determine la distribution des paquets a l'interieur du slot
	// In the slot l : distribution of packets
	IPTime = slotDuration / counterPacket;  // in seconds

	for (int i = 0; i < (counterPacket - 1); i++) {
    		
    		if( playable ) {
				fichierReq.write((IPTime*1000) + "\t" + packet + "\n");
    		}
    		else {             
				fichierReq.write( "date : t + " + MyNumberFormat.myTimeFormatter.format(time) + " s \t IA: " + 
				MyNumberFormat.myIAFormatter.format(IPTime * 1000) + " ms \t size: " + MyNumberFormat.mySizeFormatter.format(packet) + " Bytes \t rate: " + 
				MyNumberFormat.myDebitFormatter.format( (((counterPacket - 1) * packet + lastPacket) * 8 / slotDuration / 1000)) + " kb/s \n");
    		}	
				time += IPTime;
	}

	if( playable ) {
		fichierReq.write( (IPTime*1000) + "\t" + lastPacket + "\n"); 
	}
	else {
		fichierReq.write("date : t + " + MyNumberFormat.myTimeFormatter.format(time) + " s \t IA: " + MyNumberFormat.myIAFormatter.format(IPTime * 1000) + 
		" ms \t size: " + MyNumberFormat.mySizeFormatter.format(lastPacket) + " Bytes \t rate: " + 
		MyNumberFormat.myDebitFormatter.format( ((counterPacket - 1) * packet + lastPacket) * 8 / slotDuration / 1000) + " kb/s \n");
	}
		time = startTimeSlot + slotDuration;
		
		  } //end of if ( slotNew[l] != 0 ) {
	  } // fin de la boucle for

	fichierReq.flush();
	fichierReq.close();
			  
	  double debit = nbBytesTotal / time; 
	 // Compute theoritical throughput
		  double debitTheo = 0;
		  for (int i=0;i<nbDifferentFlow;i++) {
		 	debitTheo += model.getSpecificWeight(i)*((GenericContinuousDistribution)( model.getSpecificModel(i).getParameter(2))).mean() / slotDuration;  // in bytes per seconds
		  }
		  //System.out.println(nbBytesTotal);
		short unitDocument = unit;
		int docSize = nbBytesTotal;        
        
		  while ( docSize > 10000) {
		  	unitDocument++;
			docSize /= 1000;
		  }
		  
		  while( debit > 10000) {
		  	unit++;
		  	debit /= 1000;
		  	debitTheo /= 1000;
		  }
		  
		  
		if(!playable) {
		String messageInfo ="";        
			  messageInfo =  new String("Taille du document transmit: " + docSize  + " " + getUnit(unitDocument) + "Bytes.\n"
			  + "Le debit applicatif moyen est de: " +  MyNumberFormat.myDebitFormatter.format(debit) + " " + getUnit() + "Bytes/s, soit : " + MyNumberFormat.myDebitFormatter.format(debit * 8) + " " + getUnit() + "b/s.\n"
			  + "Debit applicatif theorique: " +  MyNumberFormat.myDebitFormatter.format(debitTheo) + " " + getUnit() + "Bytes/s.");
			//		   }
//		  else if(unit==KILOBYTES) {
//			  messageInfo =  new String("Taille du document transmit: " + nbBytesTotal / 1000 + " MBytes.\n"
//			  + "Le debit applicatif moyen est de: " +  myDebitFormatter.format( debit ) + " kB/s, soit : " + myDebitFormatter.format(debit * 8 / 1000) + " Mb/s.\n"
//			  + "Debit applicatif theorique: " +  myDebitFormatter.format( /**((GenericContinuousDistribution)(model.getParameter(2))).mean() / slotDuration * 8 / 1000 */ 0) + " Mb/s.");	
//		  }    	
//		  else { messageInfo = new String("No information available"); }
        
		MessageManager.informationMessage(messageInfo,graphic);
		}
		
		MessageManager.informationMessage("File created as: " + resultFile.getCanonicalPath(), graphic);
	
		return debit;
		  
	  } // fin de videoScenarioGeneration(String,int)
 

	/**
	 * To determine which source type send packets at one moment
	 * We define this method because flows have different characteristics notably packet size
	 * @return int : a sourceType number ( between 1 and flowsNumber(=> number of different sources) )
	 */
		public int sourceTypeChoice() {
			double prob = model.getSpecificWeight(0) / (double)flowsNumber; // on normalise le poids des distributions
			double u = random.nextDouble();
			int ans = 1; // it's the source type number: ans=1 <=> it's the first source type : sigma1
			while (ans <= nbDifferentFlow) {
				if (u <= prob) {
					break;
				} 
				else {
					prob += model.getSpecificWeight(ans)  / (double)flowsNumber; // on normalise le poids  des distributions
					ans++;
				}
			}
			return ans;
		}


/**
 *  Returns a String which discribes the different flows as:
 * 	 <i>model name or document name(if it exists)</i>	(<i>source number</i>)
 * @return a string = testName
 */
public String getDocumentNames() {
	String reqDoc = "";
	for (int i=0;i<nbDifferentFlow;i++ ) {
		reqDoc += model.getSpecificModel(i).getName() + " (" + model.getSpecificWeight(i) + "), ";
	}
	return reqDoc;
}

/**
 * Returns throughput of aggregatedFlow in kb/s
 * @return a double : this value is tranform to obtain a throughput in kb/s
 */
public double getTheoriticalAggregatedFlowRate() throws IOException { 
	if ( realRate==0 ) {
		flowStatisticsiGeneration(); // ici le nom du fichier n'a que peut d'importance
	}
	double ans = 0;
		
	if(unit == BYTES) {
		ans = realRate/1000*8; // transform B => kb
	}
	else if (unit == KILOBYTES) {
		ans = realRate*8; // transform kB => kb
	}
	else if (unit == MEGABYTES) {
		ans = realRate*1000*8; // transform MB => kb
	}
	else if (unit == GIGABYTES) {
		ans = realRate*1000000*8; // transform GB => kb
	}
	return ans; 
}

/**
 * Returns theoritical transmitted data amount during the generation phase in MB
 * @return a double : this value is tranform to obtain an amount in MB
 */
public double getTheoriticalTransmittedDataAmount() throws IOException { 
	if ( realRate==0 ) {
		flowStatisticsiGeneration(); // ici le nom du fichier n'a que peut d'importance
	}
	double ans = 0;
		
	if(unit == BYTES) {
		ans = realRate/1000000*simulationDuration/1000; // transform B => MB
	}
	else if (unit == KILOBYTES) {
		ans = realRate/1000*simulationDuration/1000; // transform MB => MB
	}
	else if (unit == MEGABYTES) {
		ans = realRate*simulationDuration/1000; // transform MB => MB
	}
	else if (unit == GIGABYTES) {
		ans = realRate*simulationDuration; // transform GB => MB
	}
	return ans; 
}


/**
 * Returns theoritical transmitted data amount during the generation phase (in MBytes)
 * @return a data amount in MBytes
 * @throws IOException
 */
public int getTheoriticalTransmittedDataAmountBis() throws IOException { 
	
	int ans = 0;
	int[] data = dataPerSlotDistribution();  // data in Bytes
	for (int i=0;i<N;i++) {
		ans += data[i];
	}
	
	return ans/1000000; 
}

public int getTheoriticalTransmittedDataAmountTest( int[] slot ) throws IOException { 
	int ans = 0;
	for (int i=0;i<slot.length;i++) {
		ans += slot[i];
	}
	return ans; 
}

public MGInfAggregatedFlowModel getModel() { return model; }

public int[] getFlowsTypeDuration() { return flowsDuration; }

public void setModel( MGInfAggregatedFlowModel mafm) { this.model = mafm; }

public void setSlotDuration(double duration) { this.slotDuration = duration; }

public void setFlowTypeNumber( int nbDifferentFlow ) { this.nbDifferentFlow = nbDifferentFlow; }
public int getFlowTypeNumber() { return nbDifferentFlow; }

public void setFlowsTypeDuration( int[] durations) { this.flowsDuration = durations; }

public void setFlowsTypeCodec( VideoCodec[] codecs ) { this.codecs = codecs; }

public void setFlowsTypeSlotDuration( double[] durations ) throws IllegalArgumentException {
if( durations.length == nbDifferentFlow ) {
	this.slotsDuration = durations;
}
else { throw MyExceptionDefinitions.PARAMETERS_TABLE_SIZE; }
}

public void setSourceNumber( ) { this.flowsNumber = model.getSourceNumber(); }

public void setSourceStartingTime( int[] startingTime ) throws IllegalArgumentException {
	if( startingTime.length == flowsNumber ) {
		 this.startingTime = startingTime;
	}
	else { throw MyExceptionDefinitions.PARAMETERS_TABLE_SIZE; }
}


public void setUnits(short[] units) { this.units = units; }

public void setPacketSize( ) {
	this.packetSize = new int[nbDifferentFlow];
	for (int i=0;i<nbDifferentFlow;i++ ) {
		packetSize[i] = (int)((ConstantDistribution)model.getSpecificModel(i).distributionPacketSize).getParam(0);
	}
}

public void setDurations( ) { 
	this.simulationDuration = simulationDurationComputation();
	this.tpsSimul = (int)Math.round(simulationDuration/1000/60);	 // transform ms in mn
}

public void setSingleFlowCreator() throws IOException {
	this.mginfGeneration = new MGInfGeneration[nbDifferentFlow];
	createSingleFlowGenerator();
}


	
	/**
	* Methode: Main pour tester la classe
	*/
	 public static void main(String[] args) throws IOException {
	 		
		System.out.println("Start: " + new Date().toString());
//	 	VideoDefinition[] vds1 = { BeanLow, JurassicParkLow };
//	 	int[] weight1 = { 2, 3 };
//	 	int[] startingTime1 = { 0, 5, 5, 10 , 15 };
//	 	short[] unites1= { 0, 0}; // pour dire que c'est en Bytes 
//	 		
//	 	MGInfAggregatedFlowGeneration ag1 = new MGInfAggregatedFlowGeneration(vds1, weight1, startingTime1, unites1, "aggVideo", false);
//	 	//int[] data = ag1.dataPerSlotDistribution();
//		//ag1.scenarioGeneration("fripotin",5000);
//		ag1.adaptDataPerSlotDistribution();
//		System.out.println("slot number: " + ag1.slotsNumber[0] +  " , "+ ag1.slotsNumber[1] + " new: "+ ag1.N);
//		System.out.println("slot duration: " + ag1.slotsDuration[0] +  " , "+ ag1.slotsDuration[1] +  " , "+ ag1.flowsDuration[0] +  " , "+ ag1.flowsDuration[1] +  " , " + ag1.slotDuration + " new: "+ ag1.getNewSlotReference());
//		
//		VideoDefinition[] vds2 = { JurassicParkLow };
//		int[] weight2 = { 1 };
//		int[] startingTime2 = { 0 };
//		short[] unites2 = {0};
//		
//		MGInfAggregatedFlowGeneration ag2 = new MGInfAggregatedFlowGeneration(vds2, weight2, startingTime2, unites2, "aggVideo2", false);
//		ag2.scenarioGeneration("fripotin",5000);
//		
		VideoDefinition[] vds3 = { BeanLow, JurassicParkHigh, BeanMedium};
		int[] weight3= { 2, 10, 1};
		int[] startingTime3 = { 0, 1000, 1000, 1000, 1000, 1000, 1000, 1000,  1000, 1000, 1000, 2000, 5000 };//, 10 , 15, 50 };
		short[] unites3= { 0, 0, 0}; // pour dire que c'est en Bytes 
	 		
		MGInfAggregatedFlowGeneration ag3 = new MGInfAggregatedFlowGeneration(vds3, weight3, startingTime3, unites3, "aggVideo3", false);
		//System.out.println("slot number: " + ag3.slotsNumber[0] +  " , "+ ag3.slotsNumber[1] +  " , "+ ag3.slotsNumber[2]+ " new: "+ ag3.N);
		//System.out.println("slot duration: " + ag3.slotsDuration[0] +  " , "+ ag3.slotsDuration[1] +  " , "+ ag3.slotsDuration[2]+  " , "+ ag3.flowsDuration[1] +  " , "+ ag3.flowsDuration[2] +  " , " + ag3.slotDuration + " new: "+ ag3.getNewSlotDurationReference());
		//ag3.dataPerSlotDistribution();
//			System.out.println("" + ag3.getTheoriticalTransmittedDataAmountTest(ag3.slotDurationStandardization()[0]));
//			System.out.println("" + ag3.getTheoriticalTransmittedDataAmountTest(ag3.slotNumberStandardization()[0]));
//			
//			System.out.println(ag3.getTheoriticalTransmittedDataAmountTest(ag3.slotDurationStandardization()[1]));
//			System.out.println(ag3.getTheoriticalTransmittedDataAmountTest(ag3.slotNumberStandardization()[1]));
//			
//			System.out.println(ag3.getTheoriticalTransmittedDataAmountTest(ag3.slotDurationStandardization()[2]));
//			System.out.println(ag3.getTheoriticalTransmittedDataAmountTest(ag3.slotNumberStandardization()[2]));
//	
	
		//System.out.println(	ag3.getTheoriticalTransmittedDataAmountTest( ag3.dataPerSlotDistribution()));
//				
		//System.out.println("\n Amount " + ag3.getTheoriticalTransmittedDataAmountBis() + " MB");
		//System.out.println(ag3.tota()[0].length +  " , "+  ag3.tota()[1].length  +  " , "+  ag3.tota()[2].length);
		//System.out.println(ag3.tota1()[0].length +  " , "+  ag3.tota1()[1].length  +  " , "+  ag3.tota1()[2].length);
		//System.out.println(ag3.dataPerSlotDistribution().length );
		//System.out.println(ag3.dataPerSlotDistribution()[0] + " , "+ ag3.dataPerSlotDistribution()[1]+ " , "+ ag3.dataPerSlotDistribution()[2] );
		ag3.scenarioGeneration();
		
		
		// Partie destiner a mettre en forme le fichier resultat
		//String rfnComplete = genericGenerator[i].getResultFilePath();
		String rfnComplete = ag3.getResultFileName();
		System.out.println("SCENARIO NAME: " + rfnComplete ); //DEBUG
		String tmp = "scenario.tmp";
		String[] rfn = rfnComplete.split("/", 30); //on suppose qu'on n'a pas plus de 30 "/" pour definir le path du chemin
		int rfnLength = rfn.length;
		// TODO verifier ce bout de code
		if (rfnLength != 1) {
			//MyNumberFormat.timeSizeFormatage(rfn[rfnLength-1],rfn[rfnLength-1]+".scf");
			MyNumberFormat.timeSizeFormatage(tmp, rfn[rfnLength - 1]);
		} 
		else {
			//MyNumberFormat.timeSizeFormatage(rfnComplete, rfnComplete+".scf");
			MyNumberFormat.timeSizeFormatage(tmp, rfnComplete);
		}

		
		//ag3.statisticsGeneration();
		//MyNumberFormat.timeSizeFormatage("aggVideo3","aggVideo3Format" );
		//System.out.println("Debit " + ag3.flowScenariiGeneration("fripotin", 5000) );
		//System.out.println("\n Debit " +ag3.getTheoriticalAggregatedFlowRate() );
		
		//System.out.println("\n Amount " +ag3.getTheoriticalTransmittedDataAmount() + " MB");
		
//		SpecificObjectCreation.printArray(3,ag3.transformStartingTimeInSlotsNumber());
//		
//		for (int i=0;i<ag3.nbDifferentFlow;i++) {
//			System.out.println( "1ere transfo flux "  + i + " : " );
//			 SpecificObjectCreation.printArray(10,ag3.slotDurationStandardization()[i]) ;
//			System.out.println( "2eme transfo flux "  + i + ": ");
//			SpecificObjectCreation.printArrayPart(10415, 10430, ag3.slotNumberStandardization()[i]);
//		}
		
		System.out.println("End: " + new Date().toString());
	 }
	
}
