package multimedia.generation;

import java.io.IOException;


import utils.MyTableOperation;
import utils.definitions.MyExceptionDefinitions;
import multimedia.*;
import multimedia.codec.VideoCodec;
import multimedia.model.*;
/**
 * @author mladoue
 * @version 0.1, 21/05/03
 */
public class MGInfComplexAggregatedFlowGeneration extends MGInfAggregatedFlowGeneration {
	
	private VideoDefinition[] vds;

	public MGInfComplexAggregatedFlowGeneration() {
		super();
	}
	
	public MGInfComplexAggregatedFlowGeneration ( MGInfAggregatedFlowModel model, VideoCodec[] codecs, int[] flowsDuration, int[] startingTime, short[] units,  String fileName, boolean graphic)  throws IOException {
		
	}
	/**
	 * Defined to be used by graphics.listener.MGInfAggregatedFlowGeneratorConstructor
	 * WARNING: Could be used only if all flows have the same slots number with the same duration
	 * @param vds	a table of VideoDefinition => length :  nbDifferentFlow
	 * @param weights	a table of int => length :  nbDifferentFlow
	 * @param units	a short : BYTES or KILOBYTES...
	 * @param fileName	Scenario file name
	 * @param graphic
	 * @throws IOException
	 */
	public MGInfComplexAggregatedFlowGeneration ( VideoDefinition[] vds, int[] weights, short unit, String fileName, boolean graphic)  throws IOException {
		
		super(vds, weights, unit, fileName, graphic);
		
		this.vds = vds;
		specificParametersInitialization();
	}



	protected void specificParametersInitialization() throws IOException {
		
		this.slotDuration = slotsDuration[0];  // in seconds;
		// here simulation duration is fixed to flow duration (we don't "take into account" starting time)
		this.simulationDuration = flowsDuration[0]*60*1000;  // in ms!!!!
		this.tpsSimul = flowsDuration[0];
		// As we take into account starting time to compute simulation duration => N = slotsNumber[i]
		setSlotNumber();
		
		setSourceStartingTime();
	}
	
	/**
	 *  Returns a String which discribes the different flows as:<BR>
	 * 	 <i>model name or document name(if it exists)</i>	(<i>source number</i>)
	 * @return a string = testName
	 */
	public String getDocumentNames() {
		String reqDoc = "";
		for (int i=0;i<nbDifferentFlow;i++ ) {
			reqDoc += vds[i].getName() + " (" + this.getModel().getSpecificWeight(i) + "), ";
		}
		return reqDoc;
	}
	
	
	/**
	 *   Initial data distribution : [ d1 | d2 | d3 | d4 | d5 | d6 | d7 ]  and starting time of this source is 4 (<=>slot d4 )<BR>
	 *   Final data distribution : [ d5 | d6 | d7 | d1 | d2 | d3 | d4 ] 
	 * @return an 2D int table
	 * @throws IOException
	 */
	protected int[][] circularDataPerSlotPerFlowTypeDistribution() throws IOException {
		
		int[][] newData = new int[flowsNumber][N+1];  // size : [ flowsNumber ] x [ N+1 ] 
		int[][] data = dataPerSlotPerFlowType();  // size : [ nbDifferentFlow ] x [ N+1 ]
		
		newData = MyTableOperation.circularPermutation(startingTime, data);
	
		return newData;
	}
	
	
	/**
	 * <p>
	 * This method permit to compute data per slot distribution for all sources
	 * After we transform the result according to unit and number of source of this type
	 * 	=> ex: if unit: kiloBytes -> Bytes and if there is 2 sources of this type data[i] and data[i+1= olddata[i]
	 * </p>  
	 * @return a new time slots vector which represents the amount of data which has to be generated for this type of source (i.e this flow)<br>
	 * 			size : [ flowsNumber ] x [ slotsNumberI ]
	 * @throws IOException
	 */
	public int[][] dataPerSlotPerFlowType() throws IOException {
		int[][] data = new int [flowsNumber][];  // size [ nbDifferentFlow ] x [ slotsNumberI ] <=> [ flowsNumber ] x [ NI +1 ] 
		
		int cmpt=0;
		for ( int i=0; i<nbDifferentFlow; i++) {
			// Iterate to each sources of this flow.
			for (int k=0; k < this.getModel().getSpecificWeight(i); k++) {
				// we get the dataSlotDistribution for each type flow => but it represents only one source in the flow so we have to take weight of each flows into account
				data[cmpt] = MGInfGeneration.dataPerSlotDistributionTransformation( vds[i].getModel(), N);

				// Transform data amount for each flow according to its unit(bytes,...)
				// if unit is BYTES do nothing else tranform data amount unit in BYTES (kiloBytes -> Bytes or megaBytes -> Bytes)	
				if( units[0] != BYTES ) {
					for ( int j=0;j<nbDifferentFlow;j++) {	
						data[cmpt][j] *= Math.pow(10, 3*units[0]);
					}
				}
				cmpt++;
			}
		}
		return data;
	}

	
	/**
	 * <p>
	 * You could use this method only if you have for all flow type the same number of slots and the same duration!!!
	 * We will sum data of each flow with a circular distribution to take into account starting time of each flow.
	 * That means:
	 * <ul>
	 * 	<li> the test duration will be equal to each flow duration => flowsDuration[0].
	 * 	<li> slotDuration = slotsDuration[0];
	 * 	<li> N = slotsNumber[0] 
	 * </ul>
	 * </p>
	 * Sums all data amount of sources slot per slot 
	 * @return a table of int : size slotsNumber[0]
	 */
	public int[] dataPerSlotDistribution() throws IOException {
	  	
		int[] dataAmount = new int[N+1]; 	// size : N
		int[][] data = circularDataPerSlotPerFlowTypeDistribution();	 // size [ nbDifferentFlow ] x [ slotsNumber ] <=> [ nbDifferentFlow ] x [ N+1 ]
		
		int cptSlot = 0, cpt=0;

		//	We create a temporary table to stock  sum of data of each flow with a slot number of newSlotNumber
		// 	slots vector characteristics: [ newSlotNumber, slotDurationReference] 
		for( int i=0;i<data[0].length;i++) {
			for (int j=0;j < flowsNumber;j++) {
				dataAmount[i] +=  data[j][i];	
			}
		}
		 return dataAmount;
	 }	

	/**
	 * Conditon: TestParameters.hasIntTableSameValues(slotsNumber) && TestParameters.hasDoubleTableSameValues(slotsDuration)<BR>
	 * Creates a random table with value(in number of slots) of starting time for each source
	 * @return
	 */
	protected int[] createStartingTimeTable()  {
			int[] values = new int[flowsNumber];
			values[0] = 0; // source de reference commence a To
		
			for (int i=1;i<flowsNumber;i++) {
				values[i] = random.nextInt(N);
			}
			return values;
	}

	protected double scenarioGenericGeneration( boolean playable) throws IOException {
		return super.scenarioGenericGeneration( this.dataPerSlotDistribution(), playable);
	}


	// Method defines to initialized generator parameters
	public void setFlowsTypeDuration( VideoDefinition[] vds ) throws IllegalArgumentException {
		if( vds.length == nbDifferentFlow ) {
			for ( int i=0;i<nbDifferentFlow;i++ ) { 
				flowsDuration[i] = vds[i].getTpsSimul();
			}
		}	
		else { throw MyExceptionDefinitions.PARAMETERS_TABLE_SIZE; }
	}

	protected void setFlowsTypeCodec( VideoDefinition[] vds ) throws IllegalArgumentException {
		if( vds.length == nbDifferentFlow ) {
			for ( int i=0;i<nbDifferentFlow;i++ ) {
				codecs[i] = vds[i].getCodec(); 
			}
		}
		else { throw MyExceptionDefinitions.PARAMETERS_TABLE_SIZE; }
	}
	
	protected void setUnits(short unit) {
		units = new short[nbDifferentFlow];
		for ( int i=0;i<nbDifferentFlow;i++ ) {
			units[i] = unit; 
		}
	}
	
	/**
	 * Method createSingleFlowGenerator : To create MGInfGeneration object from model,flowsDuration,codecs...
	 * 													We could use MGInfGeneration.dataPerSlotDistributionTransformation,  MGInfGeneration.dataPerSlotDistributionComputation methods
	 * @return	MGInfGeneration[] :	as many MGInfGeneration object as nbDifferentFlow 
	 * @throws IOException
	 */
	public MGInfGeneration[] createSingleFlowGenerator() throws IOException {
		mginfGeneration = new MGInfGeneration[nbDifferentFlow];
		for (int i=0;i<nbDifferentFlow;i++) {
			mginfGeneration[i] = new MGInfGeneration( vds[i].getModel(), vds[i].getTpsSimul(), vds[i].getCodec(), 1);
			mginfGeneration[i].setUnit(units[i]);
			mginfGeneration[i].setGraphicMode(graphic);
		}
			return mginfGeneration;
	}
	
	public void setFlowsTypeSlotDuration() {
		slotsDuration = new double[nbDifferentFlow];
		double value = mginfGeneration[0].slotDurationComputation();
		for ( int i=0;i<nbDifferentFlow;i++ ) {
			slotsDuration[i] = value; 
		}
	 }
	
	/**
	 * To set flow duration, codec, unit of each type flow and video definition of each flow
	 * @param vds
	 */
	public void setFlowsTypeDescription( VideoDefinition[] vds, short unit ) throws IllegalArgumentException {
		if( vds.length == nbDifferentFlow ) {
			this.vds = vds;
			flowsDuration = new int[nbDifferentFlow];
			codecs = new VideoCodec[nbDifferentFlow];
			units = new short[nbDifferentFlow];
			
			for ( int i=0;i<nbDifferentFlow;i++ ) {
				flowsDuration[i] = vds[i].getTpsSimul();
				System.out.println("---> Duration "+flowsDuration[i]);
				codecs[i] = vds[i].getCodec();
				units[i] = unit; 
			}
		}
		else { throw MyExceptionDefinitions.PARAMETERS_TABLE_SIZE; }
	}
	
	public void setDurations() {
		this.simulationDuration = flowsDuration[0]*60*1000;  // in ms!!!!
		this.tpsSimul = flowsDuration[0];
	}
	
	public void setSourceStartingTime( ) { this.startingTime = createStartingTimeTable(); }
	
	public void setSlotNumber( ) { this.N = (int) Math.round( flowsDuration[0]*60 /slotDuration); }
	
	
	public void setParameters( VideoDefinition[] vds, int[] weights, short unit, String fileName, boolean graphic ) throws IOException {
		setFlowsTypeDescription( vds, unit );	// to set vds, flowsDuration, codecs, units
		setDurations();	// to set tpsSimul	(need: flowsDuration)
		setUnit( unit );	// to set unit
		setGraphicMode( graphic );	// to set graphic!!
		setPacketSize( );	// to set packetSize[]  (need: nbDifferentFlow, vds[])
		setSingleFlowCreator();	// to set MGInfGeneration[]	(need: units[], vds[])
		setFlowsTypeSlotDuration(); // to set slotsDuration	(need: MGInfGeneration[], nbDifferentFlow)
		setSlotDuration( slotsDuration[0]);	// to set slotDuration	(need: slotsDuration[])
		setSourceNumber();	// To set flowsNumber (need: model)
		setSlotNumber();	// To set N	(need: flowsDuration, slotDuration)
		setSourceStartingTime();	// to set startingTime[] (need: N, flowsNumber)
		setResultFileName( fileName );	// to set fileName
		specificParametersInitialization();
	}

	public static void main(String[] args) throws IOException {
		
		VideoDefinition[] vds3 = { BeanLow, JurassicParkLow, BeanMedium};
		int[] weight3= { 3, 2, 4};
		//int[] startingTime3 = { 0, 1000, 5000 };//, 10 , 15, 50 };
		short[] units = { 0, 0, 0};
		
		multimedia.model.MGInfModel[] models = new multimedia.model.MGInfModel[3];
		
		for (int i=0;i<3;i++) {
			models[i] = vds3[i].getModel();
		}
	 	
		multimedia.model.MGInfAggregatedFlowModel mafm = new multimedia.model.MGInfAggregatedFlowModel();
		mafm.setParamsCount(2);
		mafm.setName("MGInfComplexAggregatedFlow");
		mafm.setFlowTypeNumber( 3 );
		mafm.setModels( models );
		mafm.setWeights( weight3 );
		mafm.setSourceNumber();
		
		MGInfComplexAggregatedFlowGeneration mginfcomplexAggregatedGenerator = new MGInfComplexAggregatedFlowGeneration();

		//mginfcomplexAggregatedGenerator.flowsNumber = 3;

		mginfcomplexAggregatedGenerator.setModel( mafm );
		mginfcomplexAggregatedGenerator.setFlowTypeNumber( 3 );
		mginfcomplexAggregatedGenerator.setParameters(vds3, weight3, (short)0, "test", false);
		
	 	
//		System.out.println("Model: " + models[0].getName() + " " +  models[0].getDistributionTest().getParam(0));	 // DEBUG
//		System.out.println("Flows number: " + mginfcomplexAggregatedGenerator.getFlowTypeNumber());	 // DEBUG
	 	
	 	
		mginfcomplexAggregatedGenerator.statisticsGeneration();
	 	
	
		//MGInfComplexAggregatedFlowGeneration ag3 = new MGInfComplexAggregatedFlowGeneration(vds3, weight3, (short)0, "aggVideo3", false);
		
		//System.out.println((ag3.dataPerSlotPerFlowType()[0]).length );
		//SpecificObjectCreation.print2DArray(15, ag3.dataPerSlotPerFlowType());
		//SpecificObjectCreation.print2DArray(15, ((MGInfAggregatedFlowGeneration)ag3).dataPerSlotPerFlowType());
		//SpecificObjectCreation.print2DArray(15, ag3.circularDataPerSlotPerFlowTypeDistribution());
		//SpecificObjectCreation.print2DArrayPart(118,125, ag3.circularDataPerSlotPerFlowTypeDistribution());
		
		//ag3.scenarioGeneration("fripotin",5000);
	}
}
