/**
 * OnOffHomogeneousAggregatedFlowModel : Generation de trafic voix pour N sources => flux aggreges<BR>
 *                            					 				Ces flux doivent etre identiques => sources homogenes.
 *
 * 
 * Methode utilisee pour realiser l'aggregation est l'IDC matching:<BR>
 *
 * Pon = beta / (alpha+beta)<BR>
 * M:etat du taux d'emission moyen, avec 0<M<N, M = partie entiere(N*Pon)<BR>
 *    => N >= (alpha + beta) / beta<BR>
 *
 * @author maria ladoue
 * @version 0 , 25/09/02 
 */


package multimedia.model;

import java.lang.Math;


import statistic.distribution.*;
import statistic.*;
import multimedia.codec.*;

public class OnOffHomogeneousAggregatedFlowModel extends GenericAggregatedModel {
    
    protected int cNb; // Nombre de source => nb de sources de ce type(i.e homogene) a genere
    protected OnOffModel typeModel;
    
    /**
     *In this case OnOff model is markovian
     * @param srcNumber it defines the source number of this OnOff modelName type
     * @param modelName it defines the Markovian OnOff modelName
     * @throws IllegalArgumentException
     */
    public OnOffHomogeneousAggregatedFlowModel( int srcNumber, OnOffModel model) throws IllegalArgumentException {
		this.cNb = srcNumber;
		this.typeModel = model;
		if ( srcNumber >= (int)(Math.floor( (getAlpha()+getBeta())/getBeta()))) {
		}
		else
			throw new IllegalArgumentException("OnOffHomogeneousAggregate.constructor : N >= floor((alpha+beta)/alpha), " +   srcNumber + " >= " + (int)(Math.floor( (getAlpha()+getBeta())/getBeta()))); 
    }


	/**
	 * To create an MMPP-2 Model which represents aggregation of N homogeneous On-Off 
	 * @return MMPP2Model
	 */
     public MMPP2Model approximation() { 

	 MMPP2Model approximateModel;
	 double Pon = this.getParamOn();
	 int m = this.getM();
	 double lambda = this.getLambda();
	 double aInf = this.getAInfinite();
	 double piUL = 0, piOL = 0;
	 double rateUL = 0, rateOL = 0;

	 for (int i=0;i<=m;i++) {
	     piUL += computePIi(i,Pon);
	 }

	 for (int i=m+1;i<=cNb;i++) {
	     piOL += computePIi(i,Pon);
	 }

	 double lambdaUL = this.computeLambdaLoad( Pon, lambda, piUL, 0, m);
	 double lambdaOL = this.computeLambdaLoad( Pon, lambda, piOL, m+1, cNb);


	 rateUL = ( 2 * (lambdaOL-cNb*Pon*lambda) * Math.pow(cNb*Pon*lambda-lambdaUL,2) ) /
		 ((lambdaOL-lambdaUL)*cNb*Pon*lambda*(aInf-1d) );
	  
	 rateOL = ( 2 * Math.pow(lambdaOL-cNb*Pon*lambda,2) * (cNb*Pon*lambda-lambdaUL) ) /
		 ((lambdaOL-lambdaUL)*cNb*Pon*lambda*(aInf-1d) );
	

	 approximateModel =  new MMPP2Model(typeModel.getEtatDepart(),rateOL,rateUL,lambdaUL,lambdaOL,typeModel.getCodec()) ;

	 return approximateModel;
	}
  

	/**
	 * Method computeLambdaLoad.
	 * @param Pon
	 * @param lambda
	 * @param PISum
	 * @param lowerBorder
	 * @param upperBorder
	 * @return double
	 */
    public double computeLambdaLoad(double Pon,double lambda, double PISum,int lowerBorder,int upperBorder) {
	double ans=0;
	for (int i=lowerBorder;i<=upperBorder;i++) {
	  ans  += i * computePIi(i,Pon);
	}
	return ((lambda/PISum)*ans);
 
    }
 
  
    public double computePIi(int i,double Pon) {
	return (  (Mathematic.factorial((long)cNb) / ( Mathematic.factorial((long)i) * Mathematic.factorial((long)(cNb-i)))) * 
		 Math.pow(Pon,i) *  Math.pow( 1 - Pon , cNb - i ) );
    }


    /**
     * getParamOn : to calculate Pon = beta/(alpha + beta)
     *  where alpha = 1/Ton  and beta = 1/Toff
     */
    public double getParamOn() {
	return getBeta()/(getAlpha() + getBeta());
    }

    /**
     * getM : to calculate M = floor(srcNumber*Pon)
     */
    public int getM() {
	return (int)Math.floor( this.getSourceNumber() * (this.getParamOn()) );
    }
    
    public double getAlpha() { return typeModel.getParam(0,"alpha");   }

    public double getBeta() { return typeModel.getParam(1,"beta");  }

    /**
     * getLambda : to calculate lambda = 1/T
     *   where T = inter-packet time
     */
    public double getLambda() {
	return 1d/((typeModel.getCodec()).getTpsIP());
    }


    public double getAInfinite() {
	return ( (1d-Math.pow((1d-getAlpha()/getLambda()),2))/Math.pow(getAlpha()/getLambda() + getBeta()/getLambda(),2));
    }

    public int getSourceNumber() {
	return cNb;
    }

    /**
     * Methode: Main pour tester la classe
     */

    public static void main (String[] args) {	
	
	AudioCodec codec1 = new AudioCodec(200,0.03);
	OnOffModel m2 = new OnOffModel(1,new ExponentialDistribution(2.8409),new ExponentialDistribution(1.5385),codec1);
	OnOffModel m1 = new OnOffModel(0,new ExponentialDistribution(1d/0.35),new ExponentialDistribution(1d/0.65),codec1);
	OnOffHomogeneousAggregatedFlowModel cm1 = new OnOffHomogeneousAggregatedFlowModel(100,m1);


	System.out.println("Alpha " + cm1.getAlpha());
	System.out.println("Beta " + cm1.getBeta());
	System.out.println("lambda " + cm1.getLambda());
	System.out.println("Pon " +cm1.getParamOn());
      	System.out.println("M " + cm1.getM());
	System.out.println("cNb " + cm1.getSourceNumber());
	System.out.println(Mathematic.factorial(cm1.getSourceNumber()));

	//for (int i=0;i<10;i++) {
	//  System.out.println( (Mathematic.factorial(cm1.getSourceNumber()) / ( Mathematic.factorial(i) * Mathematic.factorial(cm1.getSourceNumber()-i))));
	//}
	MMPP2Model model = cm1.approximation();

	//System.out.println("piO " + cm1.computePIi(0,cm1.getParamOn()));
	//System.out.println("pi1 " + cm1.computePIi(1,cm1.getParamOn()));
	//System.out.println("pi2 " + cm1.computePIi(2,cm1.getParamOn()));
 
	System.out.println("lambda1 " + model.getParam(0,"lambda"));
	System.out.println("lambda2 "  + model.getParam(1,"lambda"));

	
	System.out.println("aInf "+ cm1.getAInfinite());

	System.out.println("alpha " + model.getParam(1,"alpha"));
	System.out.println("beta " + model.getParam(0,"beta"));
	System.out.println("N>= " + (int)Math.floor( (cm1.getAlpha()+cm1.getBeta())/ cm1.getBeta() ));

 
	//System.out.println(cm1.computeLambdaLoad(cm1.getParamOn(),0,cm1.getM()));  // ok
	//System.out.println(cm1.computeLambdaOverload(cm1.getParamOn()));
	//System.out.println(cm1.computeLambdaUnderload(cm1.getParamOn()));
	//System.out.println(cm1.computeAInfinite(cm1.getParamOn()));

    }

}
