/*
 * LogNormalDistribution : Definition de la loi Log Normale
 *
 * @author
 * @version 0
 */

package statistic.distribution;

import java.lang.Math;
//import JSci.maths.statistics.LognormalDistribution;

import statistic.*;

public class LogNormalDistribution extends GenericContinuousDistribution implements RandomGenerator {

    /**
     * Equation de la densite: <BR>
     *  f(x)= exp(-( (log(x)-mu) / (sqrt(2)*sigma) )^2)/(sqrt(2*pi)*sigma)<BR>
     *     where mu = mean of a normally distributed variable <BR>
     *           sigma = standard deviation of a normally distributed variable<BR>
     *           x = value at which to evaluate the function, must be > 0 <BR> 
     */
    
    public static int MU = 0;
    public static int SIGMA  = 1;

    public LogNormalDistribution () {
        paramsCount = 2;
        name = new String("LogNormal");
        paramNames = new String[paramsCount];
        paramNames[MU] = new String("mu");
        paramNames[SIGMA] = new String("sigma");
        params = new double[paramsCount];
        params[MU] = 1.0d;
        params[SIGMA] = 1.0d;
    }

    public LogNormalDistribution (double mu, double sigma) throws IllegalArgumentException {
        this();
        if (sigma > 0) {
            params[MU] = mu;
            params[SIGMA] = sigma;
        }
        else
			throw new IllegalArgumentException("LogNormalDistribution.constructor : sigma <= 0: "+ params[SIGMA] );      	
    }

    public LogNormalDistribution (double[] params) throws IllegalArgumentException {
        this();
        if (hasValidParam(params))
        	setParams(params);
        else
			throw new IllegalArgumentException("LogNormalDistribution.constructor : sigma <= 0: "+ params[SIGMA] );      	
    }

	public boolean hasValidParam(double[] desParams) {
		if (desParams[SIGMA] >0)
			return true;
		else
			return false;
	}

    public void setParams(double[] params) throws IllegalArgumentException {
        if (params.length >= paramsCount && params[SIGMA] > 0) {
            this.params[MU] = params[MU];
            this.params[SIGMA] = params[SIGMA];
        } else {
            throw new IllegalArgumentException("std dev. l.t. 0");
        }
    }

    public void setParam(int idx, double p) throws IllegalArgumentException {
        if (idx < paramsCount && (idx != SIGMA || p > 0)) {
            params[idx] = p;
        } 
	else {
            throw new IllegalArgumentException("std dev. l.t. 0");
        }
    }

    /**
     * Returns the value of the probability distribution function at x
     */
    public double pdf(double x) {
        if (x > 0) {
	    return (1.0d/(params[SIGMA] * x * Math.sqrt(2.0d * Math.PI) *
			  Math.exp(Math.pow((Math.log(x) - params[MU])/params[SIGMA], 2.0d)/2.0d)));
        } 
	else { return 0; }
    }

    /**
     * Returns the value of the cumulative distribution function at x
     */
    public double cdf(double x) {
        if (x > 0) {
            return (normalStandardCdf((Math.log(x) - params[MU])/params[SIGMA]));
        } 
	else { return 0d; }
    }

    /**
     * Return the value of Inverse Function at x 
     * @param x with x>=0 and x<=1
     */
    public double inverseCdf(double x) {
	return Math.exp(params[SIGMA]*Mathematic.invNormal(x)+params[MU]);
    }

    /**
     * Returns the mean of the distribution
     */
    public double mean() {
        return Math.exp(params[MU] + 0.5d * Math.pow(params[SIGMA], 2.0d));
    }

    /**
     * Returns the variance of the distribution
     */
    public double var() {
        return Math.exp(2 * params[MU] + Math.pow(params[SIGMA], 2.0d)) *
            (Math.exp(Math.pow(params[SIGMA], 2.0d)) - 1);
    }

    public double rand() {
        return  Math.exp(random.nextGaussian()* params[SIGMA] + params[MU]);
    }

	public String getEquation() {
		return new String("f(x)= exp(-( (log(x)-mu) / (sqrt(2)*sigma) )^2)/(sqrt(2*pi)*sigma)");
			//+"mu = mean of a normally distributed variable sigma = standard deviation of a normally " +
			//"distributed variable  x = value at which to evaluate the function, must be > 0");
	}

    /**
     * Methode: Main pour tester la classe
     */
    
    public static void main (String[] args) {

	LogNormalDistribution lnd1 = new LogNormalDistribution(4,7);
	for (int i=0;i<1000;i++) {
	    System.out.println(lnd1.inverseCdf(random.nextDouble()));
	}
	//System.out.println(lnd1.cdf(2));
    }
    
}
