/**
 * MixedDistribution : melange de distributions continues
 *              mixedDistribution = p<sub>1</sub>*ContinuousDistribution<sub>1</sub>(params) + ... + p<sub>n</sub>*ContinuousDistribution<sub>n</sub>(params)<br>
 *              Generate a random value:<br>
 *                     You define a uniform variable u on [0 1]<br>
 *                     if u <= p<sub>i</sub> then 
 *                         randomValue = ContinuousDistribution<sub>i</sub>.rand()<br>
 *
 * 
 * @author maria ladoue
 * @version 0.1, 23/09/02 
 */

package statistic.distribution;

import statistic.*;

public class MixedDistribution extends GenericContinuousDistribution implements RandomGenerator {

    protected GenericContinuousDistribution[] distribution;
    protected double[] weigthDistribution;

    /**
     * MixedDistribution
     *
     *@param distrib it defines which distributions are involved ( => weigth of each distribution is 1/distribution number )
     */

    public MixedDistribution( GenericContinuousDistribution[] distrib ) {
		paramsCount = 1;
		name = new String("Mixed");
	
		distribution = new GenericContinuousDistribution[distrib.length];
		weigthDistribution = new double[distrib.length];

		for (int j=0;j<distrib.length;j++) {
	    	distribution[j] = distrib[j];
	    	weigthDistribution[j] = 1d / distrib.length;
		}
    }
    
    
    /**
     * MixedDistribution
     *
     *@param distrib it defines which distributions are involved
     *@param weigth it defines the weigth of each distribution
     */
    public MixedDistribution( GenericContinuousDistribution[] distrib, double[] weigth) throws IllegalArgumentException {
	this(distrib);
	paramsCount = 2;
	if (distrib.length == weigth.length) {
		weigthDistribution = new double[weigth.length];
		for (int j=0;j<weigth.length;j++) {
	   	 weigthDistribution[j] = weigth[j];
		}
	}
	else { throw new IllegalArgumentException("MixedDistribution.constructor => You must have the same number of GenericContinuousDistribution ans weight: " + distrib.length + "/" + weigth.length ); }
    }

	public boolean hasValidParam(double[] desParams) {
		return true;
	}
	
    /**
     * pdf :
     *
     *@param x it's a double
     *@param the pdf(x) value of this 'mixed' distribution
     */
    public double pdf(double x) { return getDistribution().pdf(x); }

    /**
     * Returns the value of the cumulative distribution function at x
     */
    public double cdf(double x) { //return getDistribution().cdf(x);  }
		return (weigthDistribution[0]*distribution[0].cdf(x) + weigthDistribution[1]*distribution[1].cdf(x) );  	
    }

    /**
     * Return a random value in [0 1]
     */
    public double rand() {	return getDistribution().rand();  }

    /**
     * Returns the mean of the distribution
     */
    public double mean() { //return getDistribution().mean(); }
    	return (weigthDistribution[0]*distribution[0].mean() + weigthDistribution[1]*distribution[1].mean() );
    }

    /**
     * Return the value of Inverse Function at x 
     * @param x with x>=0 and x<=1
     */
    public double inverseCdf(double x) { return getDistribution().inverseCdf(x);  }
    

    /**
     * Returns the variance of the distribution
     */
    public double var() { return getDistribution().var();   }
    
    public GenericContinuousDistribution getDistribution() {

	double p = random.nextDouble();
	double currentWeigth = 0d;
	int ans = 0;

	for ( int j=0;j<distribution.length;j++) {
	    currentWeigth += weigthDistribution[j];
	    if (p < currentWeigth) {
		ans=j;
		break;
	    }
	}
	return distribution[ans];
    }
    
    public GenericContinuousDistribution[] getSpecificDistributions() { return distribution; }
    public GenericContinuousDistribution getSpecificDistribution(int distNb) { return distribution[distNb]; }
     public void setSpecificDistribution(int distNb,GenericContinuousDistribution gcd) { distribution[distNb] = gcd; }
    
    public double[] getWeights() { return weigthDistribution; }
	public void setWeights(double[] weight) { this.weigthDistribution = weight; }
	
    public double getWeight(int idx) { return weigthDistribution[idx]; }
	public void setWeight(int idx, double weight) { this.weigthDistribution[idx] = weight; }
    
    
    
    protected int getDistributionsParametersNumber() {
    	int ans =0;
    	for (int i=0;i<distribution.length;i++) {
    	ans += distribution[i].getParamsCount();
    	}
    	return ans;
    }
    
  public double[] getDistributionsParameters() { 
    	double[] params = new double[getDistributionsParametersNumber()];
    	int k = 0;
    	for (int i=0;i<distribution.length;i++) {
    		for (int j=0;j<distribution[i].getParamsCount();j++) {
    			params[k] = distribution[i].getParam(j);
    			k++;
    		}
    	}
   		return params;
    }
    
    public void setDistributionsParameter(double[] b) {
    	int k = 0;
    	for (int i=0;i<distribution.length;i++) {
    		for ( int j=0;j<distribution[i].getParamsCount();j++) {
    			distribution[i].setParam(j,b[k]);
    			k++;
    		}
    	}
   }

   public String getEquation() {
	   return new String("Mixed distribution");
   }

   /**
     * Methode: Main pour tester la classe
     */
    public static void main (String[] args) {
	LogNormalDistribution lgnd1 = new LogNormalDistribution(9.018,1.7113);
	ConstantDistribution cd1 = new ConstantDistribution(1d/1409d);
	GenericContinuousDistribution[] gd1 = {lgnd1,cd1};
	MixedDistribution md1 = new MixedDistribution(gd1);
	
	double[] coeffDistjph = { 0.286, 0.714 };
	GenericContinuousDistribution[] gdjph = { new GammaDistribution(5.879,9484.9), new LogNormalDistribution(10.547,0.461)};
	GenericContinuousDistribution djph = new MixedDistribution(gdjph,coeffDistjph);
	
	
	
	
	//System.out.println(md1.mean());
	//System.out.println(lgnd1.mean());
	//System.out.println(cd1.mean());
	//System.out.println(md1.cdf(0.002));

	//	for(int i=0;i<10;i++) {
		    //System.out.println("random mixed: " + (djph.mean()));
	//}
	
    }

}
