/**
 * NormalDistribution : Definition de la loi Normale
 *
 * @author maria ladoue
 * @version 0.1, 30/08/02 
 */
package statistic.distribution;

import VisualNumerics.math.Statistics;

import java.lang.Math;

import statistic.*;

public class NormalDistribution extends GenericContinuousDistribution implements RandomGenerator {

    /**    
     * Equation de la densite:<BR>
     *  f(x) = exp(-((x-mu) / (sqrt(2)*sigma) )^2)/(sqrt(2*pi)*sigma)<BR>
     *      where mu = mean of the normal distribution <BR>
     *            sigma = standard deviation must be<BR>
     */
	private static int MU = 0;
	private static int SIGMA = 1;

    public NormalDistribution() {
        paramsCount = 2;
        name = new String("Normal");
        paramNames = new String[paramsCount];
        paramNames[MU] = new String("mu");
        paramNames[SIGMA] = new String("sigma");
        params = new double[paramsCount];
        params[MU] = 0.0d;
        params[SIGMA] = 1.0d;
    }

    public NormalDistribution(double mu, double sigma)
                       throws IllegalArgumentException {
        this();

        if (sigma > 0) {
            params[MU] = mu;
            params[SIGMA] = sigma;
        } else {
			throw new IllegalArgumentException("PoissonDistribution.setParams : sigma <= 0: "+ params[SIGMA] );
       }
    }

    public NormalDistribution(double[] params)
                       throws IllegalArgumentException {
        this();
        if (hasValidParam(params)) 
        	setParams(params);
        else
			throw new IllegalArgumentException("NormalDistribution.constructor : sigma <= 0: "+ params[SIGMA] );      	
    }

	public boolean hasValidParam(double[] desParams) {
		if (desParams[SIGMA]  > 0)
			return true;
		else
			return false;
	}

    public void setParams(double[] params)
                   throws IllegalArgumentException {

        if (params.length >= paramsCount && params[SIGMA] > 0) {
            this.params[MU] = params[MU];
            this.params[SIGMA] = params[SIGMA];
        } else {
			throw new IllegalArgumentException("NormalDistribution.setParams : sigma <= 0: "+ params[SIGMA] );      	
        }
    }

    public void setParam(int idx, double p)
                  throws IllegalArgumentException {

        if (idx < paramsCount && (idx != SIGMA || p > 0)) {
            params[idx] = p;
        } else {
            throw new IllegalArgumentException("NormalDistribution.setParam : sigma and mu must be greater than 0.");
        }
    }

    /**
     * Returns the value of the probability distribution function at x
     */
    public double pdf(double x) {

        return (1.0d / (params[SIGMA] * Math.sqrt(2.0d * Math.PI) * Math.exp(Math.pow((x - params[MU]) / params[SIGMA], 
                                                                                      2.0d) / 2.0d)));
    }

    /**
     * Returns the value of the cumulative distribution function at x
     */
    public double cdf(double x) {

        return normalStandardCdf((x - params[MU]) / params[SIGMA]);
    }

    /**
     * Return the value of Inverse Function at x
     * @param x with x>=0 and x<=1
     */
    public double inverseCdf(double x) {

        //return params[MU] + params[SIGMA]*Mathematic.invNormal(x);
        return params[MU] + params[SIGMA] * Statistics.inverseNormalCdf(x);
    }

    public double mean() {

        return params[MU];
    }

    public double var() {

        return Math.pow(params[SIGMA], 2.0d);
    }

    public double rand() {

        return random.nextGaussian() * params[SIGMA] + params[MU];
    }
	
	public String getEquation() {
		return new String("f(x) = exp(-((x-mu) / (sqrt(2)*sigma) )^2)/(sqrt(2*pi)*sigma) where " +
			"mu = mean of the normal distribution sigma = standard deviation");
	}

    /**
     * Methode: Main pour tester la classe
     */
    public static void main(String[] args) {

        NormalDistribution nd1 = new NormalDistribution(0, 1); // Definition: NormalDistribution (double mu, double sigma)

        for (int i = 0; i < 1000; i++) {

            //System.out.println(nd1.inverseCdf(random.nextDouble()));
            //System.out.println(random.nextDouble());
            //System.out.println(nd1.rand());
        }

        System.out.println("inverseCdf 0.1 : " + nd1.inverseCdf(0.1));
        System.out.println(
                "inverseCdf 0.1 : " + Statistics.inverseNormalCdf(0.1));

        /**
       System.out.println( "inverseCdf 0.1 : " + nd1.inverseCdf(0.1));
    System.out.println( "inverseCdf 0.15 : " + nd1.inverseCdf(0.15));
    System.out.println( "inverseCdf 0.2 : " + nd1.inverseCdf(0.2));
    System.out.println( "inverseCdf 0.25 : " + nd1.inverseCdf(0.25));
    System.out.println( "inverseCdf 0.5 : " + nd1.inverseCdf(0.5));
    System.out.println( "inverseCdf 0.7 : " + nd1.inverseCdf(0.7));
    System.out.println( "inverseCdf 0.9 : " + nd1.inverseCdf(0.9));
    System.out.println( "inverseCdf 1.0 : " + nd1.inverseCdf(1)); // OK
    */
    }
}