package traffic.distributed;

/**
 * MasterNode : 
 * This class is in charged of exporting HostInformation services
 * It also starts the necessary Slave Nodes (client/server) on remote machines
 * according hostinfo.xml file.
 * It creates RemoteUDPClient on clients
 * It creates UDPServerManager on server (this class instanciate a thread UDP server).
 *
 * @author dsagnol
 * @version 0.2
 */

import java.rmi.ConnectException;
import java.net.*;
import java.rmi.*;
import java.io.*;


public class MasterNode {
    
    HostInformations infos;
    int rmiPort;
    String serviceName;
    
	/**
	 * @param serverName
	 * @param rmiPort
	 * @throws InterruptedException
	 */
    public MasterNode(int rmiPort, String serviceName, String xmlfilename) {
        try {
            this.rmiPort = rmiPort;
            this.serviceName = serviceName;
            java.rmi.registry.LocateRegistry.createRegistry( rmiPort );
            if (xmlfilename.equals(""))
                infos = new HostInformationsImpl();
            else
                infos = new HostInformationsImpl(xmlfilename);
            
            if (System.getSecurityManager() == null) {
                System.setSecurityManager( new RMISecurityManager() );
            }
            
                //  Format: "//host:port/name" => "//:port/name"
            Naming.rebind("rmi://localhost:" + rmiPort + "/" + serviceName, infos);
            System.out.println(" Server ready... Host Informations available on: " +  "rmi//localhost:" + rmiPort + "/" + serviceName);
        }
        catch (ConnectException ce) { System.err.println(ce.getMessage()); }
        catch (RemoteException re) { re.printStackTrace(); }
        catch (MalformedURLException mue) { mue.printStackTrace(); }
        catch (IOException ioe) { ioe.printStackTrace(); }
    }
    
        /**
         * Based on information read by the HostInformations object
         * create the remote Servers and clients for this scenario
         */
    public void deployApplication(OutputManagerI om) throws RemoteException {
            // First, create the servers...
            // Create a string for the programme to execute on remote host.
        try {
            String[] serverList = infos.getServerList();
            int[] serverPortList = infos.getServerPortList();
            String[] javaCmdServerList = infos.getJavaCmdServerList();
            for(int j=0; j < serverList.length; j++) {
                String accesstype=infos.getAccessTypeForServer(serverList[j], serverPortList[j]);
                	// Check if it's ipv4 or ipv6.
                	// Get client list...
                String[] clientList = infos.getClientsList(serverList[j], serverPortList[j]);
				int[] clientListPort = infos.getClientsPortList(serverList[j], serverPortList[j]);
                String execServerProg;
                if (!infos.isClientIPv4(clientList[0], clientListPort[0])) {
					execServerProg= accesstype+" "+serverList[j]+" "+javaCmdServerList[j]+" -Djava.net.preferIPv6Addresses=true "+" traffic.distributed.UDPServerManager "+InetAddress.getLocalHost().getHostName()+" "+rmiPort;          
                }
                else
                	execServerProg= accesstype+" "+serverList[j]+" "+javaCmdServerList[j]+" traffic.distributed.UDPServerManager "+InetAddress.getLocalHost().getHostName()+" "+rmiPort;
                System.out.println(execServerProg);
                Process child = Runtime.getRuntime().exec(execServerProg);

					// Take care of output stream of this Process.
				om.manageOutputStreams(child, serverList[j],""+serverPortList[j]);                  
             }
        }
        catch(IOException ioe) {
            System.out.println(ioe);
        }

            // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            // BESOIN D'UNE TEMPORISATION....
            // En effet, si les serveurs sont un peu lent, un client peut essayer de les
            // contacter alors qu'ils n'ont pas encore demarre...
            //////////////////////////////////////////////////////////////////////
        try {
            Thread.sleep(4000);
        }
        catch (InterruptedException ie) {
            System.out.println(ie);
        }

            // Second, create the clients...
        int clientsNB=infos.getClientsList().length;
        String[] clientList = infos.getClientsList();
        int [] clientPortList = infos.getClientsPortList();
        String[] javaCmdClientList = infos.getJavaCmdClientsList();
        for(int i=0; i < clientsNB; i++) {
            System.out.println("Start a client on: "+clientList[i]+":"+clientPortList[i]);
            try {
                String clientaccesstype=infos.getAccessTypeForClient(clientList[i], clientPortList[i]);
                String s;
                if (infos.isClientIPv4(clientList[i], clientPortList[i]))
                	s=clientaccesstype+" "+clientList[i]+" "+javaCmdClientList[i]+" traffic.distributed.RemoteUDPClient "+InetAddress.getLocalHost().getHostName() +" "+rmiPort;
                else
					s=clientaccesstype+" "+clientList[i]+" "+javaCmdClientList[i]+" -Djava.net.preferIPv6Addresses=true traffic.distributed.RemoteUDPClient "+InetAddress.getLocalHost().getHostName() +" "+rmiPort;
                System.out.println(s);
                Process child = Runtime.getRuntime().exec(s);
            
				// Take care of output stream of this Process.
				om.manageOutputStreams(child, clientList[i],""+clientPortList[i]);                  
                Thread.sleep(500);
            }
            catch (IOException ioe) {
                System.out.println(ioe);
            }
            catch (InterruptedException ie) {
                System.out.println(ie);
            }
        }
    }

    public static void main(String[] args) {
        
	String serviceName;
        String xmlfile;
	int rmiPort;
		
     // Usage: java traffic.distributed.MasterNode <rmi-port> <xmlhostfile> [serviceName] 
	if(args.length < 1 ) {
            System.err.println("Usage: java traffic.distributed.MasterNode <rmi-port> <xmlhostfile> [service-name]");
            return;
	}	
	
	rmiPort = Integer.parseInt(args[0]);
        if (args.length > 2)
            serviceName = args[2];
        else
            serviceName=HostInformations.name;
	
        if (args.length > 1)
            xmlfile = args[1];
        else
            xmlfile="";

	try {
            MasterNode mn = new MasterNode(rmiPort,serviceName, xmlfile);
            ConsoleOutputManager om =new ConsoleOutputManager(); 
            mn.deployApplication(om);
	} 
        catch(RemoteException re)
        {
            System.out.println(re);
        }

    }
}
